/*
  growbuf - Buffer that can always be appended to and iterator.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: growbuf.c,v 1.1 2006/04/11 18:38:28 emunson Exp $
  
  */

#include <stdlib.h>
#include <string.h>

#include "growbuf.h"

/*
 * growbuf:
 */

growbuf *
growbuf_new(size_t chunk_size)
{
	growbuf * ret = malloc(sizeof(growbuf));
	if (NULL != ret) {
		ret->data = NULL;
		ret->len = 0;
		ret->max = 0;
		ret->chunk_size = chunk_size;
	}

	return ret;
}	

void
growbuf_free(growbuf * buf)
{
	if (NULL != buf) {
		free(buf->data);
	}
	free(buf);
}

growbuf *
growbuf_append(growbuf             * buf ,
	       const unsigned char * data,
	       size_t                data_len)
{
	if (buf->len + data_len > buf->max) {
		buf->max = ((buf->len + data_len) /
			    buf->chunk_size + 1) * buf->chunk_size;
		buf->data = realloc(buf->data, buf->max);
	}

	if (NULL != buf->data) {
		memcpy(buf->data + buf->len, data, data_len);
		buf->len += data_len;
	} else {
		free(buf);
		buf = NULL;
	}

	return buf;
}

/*
 * growbuf_iter:
 */

growbuf_iter *
growbuf_iter_new(const growbuf * buf)
{
	growbuf_iter * ret = malloc(sizeof(growbuf_iter));
	ret->pos = 0;
	ret->buf  = buf;

	return ret;
}

void
growbuf_iter_free(growbuf_iter * i)
{
	free(i);
}
